@@ -1,4 +1,5 @@
 import os
+from pathlib import Path
 import re
 import sys
 import mplcursors
@@ -14,7 +15,13 @@ extensions = [
     'sphinx.ext.viewcode',
     'sphinx_gallery.gen_gallery',
 ]
-needs_extensions = {'sphinx_gallery.gen_gallery': '0.6.0'}
+_req_path = Path('../../.doc-requirements.txt')
+needs_extensions = {
+    'sphinx_gallery.gen_gallery':
+    dict(line.split('==') for line in _req_path.read_text().splitlines())[
+        'sphinx-gallery']
+
+}
 
 source_suffix = '.rst'
 exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']
@@ -35,21 +42,16 @@ pygments_style = 'sphinx'
 
 todo_include_todos = False
 
+python_use_unqualified_type_names = True
+
 # -- Options for HTML output ----------------------------------------------
 
-html_theme = 'alabaster'
+html_theme = 'pydata_sphinx_theme'
 html_theme_options = {
-    'description': 'Interactive data selection cursors for Matplotlib.',
-    'github_user': 'anntzer',
-    'github_repo': 'mplcursors',
-    'github_banner': True,
-    'github_button': False,
-    'code_font_size': '80%',
+    'github_url': 'https://github.com/anntzer/mplcursors',
 }
 html_css_files = ['hide_some_gallery_elements.css']
 html_static_path = ['_static']
-html_sidebars = {'**': ['about.html', 'navigation.html', 'localtoc.html']}
-# html_last_updated_fmt = ''  # bitprophet/alabaster#93
 
 htmlhelp_basename = 'mplcursors_doc'
 
@@ -92,8 +94,8 @@ autodoc_member_order = 'bysource'
 
 intersphinx_mapping = {
     'python': ('https://docs.python.org/3', None),
-    'matplotlib': ('https://matplotlib.org', None),
-    'pandas': ('http://pandas.pydata.org/pandas-docs/stable', None),
+    'matplotlib': ('https://matplotlib.org/stable', None),
+    'pandas': ('https://pandas.pydata.org/pandas-docs/stable', None),
 }
 
 # CustomSortKey cannot be defined *here* because it would be unpicklable as
