@@ -97,21 +97,16 @@ class ContainerArtist:
         return True  # For lack of anything better.
 
 
-class AttrArray(np.ndarray):
-    """An array subclass that can store additional attributes."""
+Selection = namedtuple(
+    "Selection", "artist target_ index dist annotation extras")
+Selection.__doc__ = """
+    A selection.
 
-    def __new__(cls, array):
-        return np.asarray(array).view(cls)
-
-
-def _with_attrs(array, **kwargs):
-    array = AttrArray(array)
-    for k, v in kwargs.items():
-        setattr(array, k, v)
-    return array
-
-
-Selection = namedtuple("Selection", "artist target dist annotation extras")
+    Although this class is implemented as a namedtuple (to simplify the
+    dispatching of `compute_pick`, `get_ann_text`, and `make_highlight`), only
+    the field names should be considered stable API.  The number and order of
+    fields are subject to change with no notice.
+"""
 # Override equality to identity: Selections should be considered immutable
 # (with mutable fields though) and we don't want to trigger casts of array
 # equality checks to booleans.  We don't need to override comparisons because
@@ -120,8 +115,13 @@ Selection.__eq__ = lambda self, other: self is other
 Selection.__ne__ = lambda self, other: self is not other
 Selection.artist.__doc__ = (
     "The selected artist.")
-Selection.target.__doc__ = (
-    "The point picked within the artist, in data coordinates.")
+Selection.target_.__doc__ = """
+The point picked within the artist, in data coordinates.
+
+:meta private:
+"""
+Selection.index.__doc__ = (
+    "The index of the selected point, within the artist data.")
 Selection.dist.__doc__ = (
     "The distance from the click to the target, in pixels.")
 Selection.annotation.__doc__ = (
@@ -131,6 +131,31 @@ Selection.extras.__doc__ = (
     "at the same time as the annotation.")
 
 
+class _Target(np.ndarray):
+    def __new__(cls, sel):
+        obj = np.asarray(sel.target_).view(cls)
+        obj._sel = sel
+        return obj
+
+    @property
+    def index(self):
+        warnings.warn(
+            "Selection.target.index is deprecated and will be removed in the "
+            "future; use Selection.index instead.")
+        return self._sel.index
+
+
+Selection.target = property(
+    _Target, doc="The point picked within the artist, in data coordinates.")
+
+
+def _with_attrs(array, **kwargs):
+    array = AttrArray(array)
+    for k, v in kwargs.items():
+        setattr(array, k, v)
+    return array
+
+
 @functools.singledispatch
 def compute_pick(artist, event):
     """
@@ -247,12 +272,10 @@ def _compute_projection_pick(artist, path, xy):
     except (ValueError, IndexError):  # See above re: exceptions caught.
         return
     else:
-        target = AttrArray(
-            artist.axes.transData.inverted().transform(projs[argmin]))
-        target.index = (
-            (argmin + dot[argmin] / ls[argmin])
-            / (path._interpolation_steps / tpath._interpolation_steps))
-        return Selection(artist, target, dmin, None, None)
+        target = artist.axes.transData.inverted().transform(projs[argmin])
+        index = ((argmin + dot[argmin] / ls[argmin])
+                 / (path._interpolation_steps / tpath._interpolation_steps))
+        return Selection(artist, target, index, dmin, None, None)
 
 
 def _untransform(orig_xy, screen_xy, ax):
@@ -293,23 +316,22 @@ def _(artist, event):
         except ValueError:  # Raised by nanargmin([nan]).
             pass
         else:
-            target = _with_attrs(
-                _untransform(  # More precise than transforming back.
-                    data_xy[argmin], data_screen_xy[argmin], artist.axes),
-                index=argmin)
-            sels.append(Selection(artist, target, ds[argmin], None, None))
+            target = _untransform(  # More precise than transforming back.
+                data_xy[argmin], data_screen_xy[argmin], artist.axes)
+            sels.append(
+                Selection(artist, target, argmin, ds[argmin], None, None))
     # If lines are visible, find the closest projection.
     if (artist.get_linestyle() not in ["None", "none", " ", "", None]
             and len(artist.get_xydata()) > 1):
         sel = _compute_projection_pick(artist, artist.get_path(), xy)
         if sel is not None:
-            sel.target.index = {
+            sel = sel._replace(index={
                 "_draw_lines": lambda _, index: index,
                 "_draw_steps_pre": Index.pre_index,
                 "_draw_steps_mid": Index.mid_index,
                 "_draw_steps_post": Index.post_index}[
                     Line2D.drawStyles[artist.get_drawstyle()]](
-                        len(data_xy), sel.target.index)
+                        len(data_xy), sel.index))
             sels.append(sel)
     sel = min(sels, key=lambda sel: sel.dist, default=None)
     return sel if sel and sel.dist < artist.get_pickradius() else None
@@ -343,11 +365,10 @@ def _(artist, event):
         offsets_screen = artist.get_offset_transform().transform(offsets)
         ds = np.hypot(*(offsets_screen - [event.x, event.y]).T)
         argmin = ds.argmin()
-        target = _with_attrs(
-            _untransform(offsets[argmin], offsets_screen[argmin], artist.axes),
-            index=inds[argmin])
-        return Selection(artist, target, ds[argmin], None, None)
-    else:
+        target = _untransform(
+            offsets[argmin], offsets_screen[argmin], artist.axes)
+        return Selection(artist, target, inds[argmin], ds[argmin], None, None)
+    elif len(paths) and len(offsets):
         # Note that this won't select implicitly closed paths.
         sels = [*filter(None, [
             _compute_projection_pick(
@@ -362,9 +383,7 @@ def _(artist, event):
         sel = sels[idx]
         if sel.dist >= artist.get_pickradius():
             return None
-        sel = sel._replace(artist=artist)
-        sel.target.index = (idx, sel.target.index)
-        return sel
+        return sel._replace(artist=artist, index=(idx, sel.index))
 
 
 @compute_pick.register(AxesImage)
@@ -386,8 +405,7 @@ def _(artist, event):
         ymin, ymax = ymax, ymin
     low, high = np.array([[xmin, ymin], [xmax, ymax]])
     idxs = ((xy - low) / (high - low) * ns).astype(int)[::-1]
-    target = _with_attrs(xy, index=tuple(idxs))
-    return Selection(artist, target, 0, None, None)
+    return Selection(artist, xy, tuple(idxs), 0, None, None)
 
 
 @compute_pick.register(Barbs)
@@ -398,10 +416,9 @@ def _(artist, event):
     ds = np.hypot(*(offsets_screen - [event.x, event.y]).T)
     argmin = np.nanargmin(ds)
     if ds[argmin] < artist.get_pickradius():
-        target = _with_attrs(
-            _untransform(offsets[argmin], offsets_screen[argmin], artist.axes),
-            index=argmin)
-        return Selection(artist, target, ds[argmin], None, None)
+        target = _untransform(
+            offsets[argmin], offsets_screen[argmin], artist.axes)
+        return Selection(artist, target, argmin, ds[argmin], None, None)
     else:
         return None
 
@@ -424,7 +441,7 @@ def _(container, event):
             if patch.contains(event)[0]}
     except ValueError:
         return
-    target = _with_attrs([event.xdata, event.ydata], index=idx)
+    target = [event.xdata, event.ydata]
     if patch.sticky_edges.x:
         target[0], = (
             x for x in [patch.get_x(), patch.get_x() + patch.get_width()]
@@ -433,7 +450,7 @@ def _(container, event):
         target[1], = (
             y for y in [patch.get_y(), patch.get_y() + patch.get_height()]
             if y not in patch.sticky_edges.y)
-    return Selection(container, target, 0, None, None)
+    return Selection(container, target, idx, 0, None, None)
 
 
 @compute_pick.register(ErrorbarContainer)
@@ -446,12 +463,12 @@ def _(container, event):
     if (sel_data and sel_data.dist < getattr(sel_err, "dist", np.inf)):
         return sel_data
     elif sel_err:
-        idx, _ = sel_err.target.index
+        idx, _ = sel_err.index
         if data_line:
-            target = _with_attrs(data_line.get_xydata()[idx], index=idx)
+            target = data_line.get_xydata()[idx]
         else:  # We can't guess the original data in that case!
             return
-        return Selection(container, target, 0, None, None)
+        return Selection(container, target, idx, 0, None, None)
     else:
         return
 
@@ -467,15 +484,17 @@ def _(container, event):
         return
     sel = compute_pick(container.stemlines, event)
     if sel:
-        idx, _ = sel.target.index
-        target = _with_attrs(
-            container.stemlines.get_segments()[idx][-1],
-            index=sel.target.index)
-        return Selection(container, target, 0, None, None)
+        idx, _ = sel.index
+        target = container.stemlines.get_segments()[idx][-1]
+        return Selection(container, target, sel.index, 0, None, None)
 
 
-def _call_with_selection(func):
+def _call_with_selection(func=None, *, argname="artist"):
     """Decorator that passes a `Selection` built from the non-kwonly args."""
+
+    if func is None:
+        return functools.partial(_call_with_selection, argname=argname)
+
     wrapped_kwonly_params = [
         param for param in inspect.signature(func).parameters.values()
         if param.kind == param.KEYWORD_ONLY]
@@ -494,8 +513,9 @@ def _call_with_selection(func):
         sel = Selection(*ba.args, **ba.kwargs)
         return func(sel, **extra_kw)
 
-    wrapper.__signature__ = Signature(
-        [*sel_sig.parameters.values(), *wrapped_kwonly_params])
+    params = [*sel_sig.parameters.values(), *wrapped_kwonly_params]
+    params[0] = params[0].replace(name=argname)
+    wrapper.__signature__ = Signature(params)
     return wrapper
 
 
@@ -513,7 +533,7 @@ def _format_coord_unspaced(ax, xy):
 @_call_with_selection
 def get_ann_text(sel):
     """
-    Compute an annotating text for a `Selection` (passed **unpacked**).
+    Compute an annotating text for an (unpacked) `Selection`.
 
     This is a single-dispatch function; implementations for various artist
     classes follow.
@@ -562,7 +582,7 @@ def _(sel):
             # to involve an arbitrary scaling).
             and artist.get_array() is not None
             and len(artist.get_array()) == len(artist.get_offsets())):
-        value = _format_scalarmappable_value(artist, sel.target.index)
+        value = _format_scalarmappable_value(artist, sel.index)
         text = f"{text}\n{value}"
     if re.match("[^_]", label):
         text = f"{label}\n{text}"
@@ -577,7 +597,7 @@ _Event = namedtuple("_Event", "xdata ydata")
 def _(sel):
     artist = sel.artist
     text = _format_coord_unspaced(artist.axes, sel.target)
-    cursor_text = _format_scalarmappable_value(artist, sel.target.index)
+    cursor_text = _format_scalarmappable_value(artist, sel.index)
     return f"{text}\n{cursor_text}"
 
 
@@ -587,7 +607,7 @@ def _(sel):
     artist = sel.artist
     text = "{}\n{}".format(
         _format_coord_unspaced(artist.axes, sel.target),
-        (artist.u[sel.target.index], artist.v[sel.target.index]))
+        (artist.u[sel.index], artist.v[sel.index]))
     return text
 
 
@@ -597,7 +617,7 @@ def _(sel):
     artist = sel.artist
     text = "{}\n{}".format(
         _format_coord_unspaced(artist.axes, sel.target),
-        (artist.U[sel.target.index], artist.V[sel.target.index]))
+        (artist.U[sel.index], artist.V[sel.index]))
     return text
 
 
@@ -608,24 +628,24 @@ def _(sel):
 
 
 @get_ann_text.register(BarContainer)
-@_call_with_selection
+@_call_with_selection(argname="container")
 def _(sel):
     return _format_coord_unspaced(
         _artist_in_container(sel.artist).axes, sel.target)
 
 
 @get_ann_text.register(ErrorbarContainer)
-@_call_with_selection
+@_call_with_selection(argname="container")
 def _(sel):
     data_line, cap_lines, err_lcs = sel.artist
     ann_text = get_ann_text(*sel._replace(artist=data_line))
-    if isinstance(sel.target.index, Integral):
+    if isinstance(sel.index, Integral):
         err_lcs = iter(err_lcs)
         for idx, (dir, has) in enumerate(
                 zip("xy", [sel.artist.has_xerr, sel.artist.has_yerr])):
             if has:
-                err = (next(err_lcs).get_paths()[sel.target.index].vertices
-                       - data_line.get_xydata()[sel.target.index])[:, idx]
+                err = (next(err_lcs).get_paths()[sel.index].vertices
+                       - data_line.get_xydata()[sel.index])[:, idx]
                 err_s = [getattr(_artist_in_container(sel.artist).axes,
                                  f"format_{dir}data")(e).rstrip()
                          for e in err]
@@ -648,7 +668,7 @@ def _(sel):
 
 
 @get_ann_text.register(StemContainer)
-@_call_with_selection
+@_call_with_selection(argname="container")
 def _(sel):
     return get_ann_text(*sel._replace(artist=sel.artist.markerline))
 
@@ -657,7 +677,7 @@ def _(sel):
 @_call_with_selection
 def move(sel, *, key):
     """
-    Move a `Selection` (passed **unpacked**) following a keypress.
+    Move an (unpacked) `Selection` following a keypress.
 
     This function is used to implement annotation displacement through the
     keyboard.
@@ -672,14 +692,13 @@ def _move_within_points(sel, xys, *, key):
     # Avoid infinite loop in case everything became nan at some point.
     for _ in range(len(xys)):
         if key == "left":
-            new_idx = int(np.ceil(sel.target.index) - 1) % len(xys)
+            new_idx = int(np.ceil(sel.index) - 1) % len(xys)
         elif key == "right":
-            new_idx = int(np.floor(sel.target.index) + 1) % len(xys)
+            new_idx = int(np.floor(sel.index) + 1) % len(xys)
         else:
             return sel
-        target = _with_attrs(xys[new_idx], index=new_idx)
-        sel = sel._replace(target=target, dist=0)
-        if np.isfinite(target).all():
+        sel = sel._replace(target_=xys[new_idx], index=new_idx, dist=0)
+        if np.isfinite(sel.target).all():
             return sel
 
 
@@ -713,19 +732,18 @@ def _(sel, *, key):
 @_call_with_selection
 def _(sel, *, key):
     ns = sel.artist.get_array().shape[:2]
-    idxs = (np.asarray(sel.target.index)
-            + {"left": [0, -1],
-               "right": [0, 1],
-               "up": {"lower": [1, 0], "upper": [-1, 0]}[sel.artist.origin],
-               "down": {"lower": [-1, 0], "upper": [1, 0]}[sel.artist.origin]}[
-                   key]) % ns
+    delta = (
+        {"left": [0, -1], "right": [0, +1], "down": [-1, 0], "up": [+1, 0]}[
+            key]
+        * np.array([-1 if sel.artist.axes.yaxis.get_inverted() else +1,
+                    -1 if sel.artist.axes.xaxis.get_inverted() else +1]))
+    idxs = (sel.index + delta) % ns
     xmin, xmax, ymin, ymax = sel.artist.get_extent()
     if sel.artist.origin == "upper":
         ymin, ymax = ymax, ymin
     low, high = np.array([[xmin, ymin], [xmax, ymax]])
-    target = _with_attrs(((idxs + .5) / ns)[::-1] * (high - low) + low,
-                         index=tuple(idxs))
-    return sel._replace(target=target)
+    target = ((idxs + .5) / ns)[::-1] * (high - low) + low
+    return sel._replace(target_=target, index=tuple(idxs))
 
 
 @move.register(ContainerArtist)
@@ -736,7 +754,7 @@ def _(sel, *, key):
 
 
 @move.register(ErrorbarContainer)
-@_call_with_selection
+@_call_with_selection(argname="container")
 def _(sel, *, key):
     data_line, cap_lines, err_lcs = sel.artist
     return _move_within_points(sel, data_line.get_xydata(), key=key)
@@ -746,7 +764,7 @@ def _(sel, *, key):
 @_call_with_selection
 def make_highlight(sel, *, highlight_kwargs):
     """
-    Create a highlight for a `Selection`.
+    Create a highlight for an (unpacked) `Selection`.
 
     This is a single-dispatch function; implementations for various artist
     classes follow.
@@ -775,6 +793,6 @@ def _(sel, *, highlight_kwargs):
     hl = copy.copy(sel.artist)
     offsets = hl.get_offsets()
     hl.set_offsets(np.where(
-        np.arange(len(offsets))[:, None] == sel.target.index, offsets, np.nan))
+        np.arange(len(offsets))[:, None] == sel.index, offsets, np.nan))
     _set_valid_props(hl, highlight_kwargs)
     return hl
